; E2 Driver for the TI-86 (Version 2.00)
; Written By Bryan Rittmeyer <bryanr@pair.com>
; Copyright (c) 1997-1999 Horizon Technologies
; http://horizon.pair.com/
;
; I've tried to make this as ZShell friendly as possible
; to facilitate a USGard/ZShell port. Many ZShell defines
; and calls are used.
;
; Many of the comments are outdated; If something seems
; a little confusing, go by the code and not the comments
;
; This code is now under the terms of the GNU Public License.
; Please see the included gpl.txt file for more information.

; Includes
#include "asm86.h"		; Main include for Asm86
#include "ti86asm.inc"		; Normal TI asm include

; Entry point
.org _asm_exec_ram
	ld (spbak),sp		; Backup stack pointer..

	ld a, $C0		; Set W1 and R1
        out (port),a		; Setup link port

	call BUSY_OFF		; Turn off run indicator

	ld a,$B			;
	ld (barpos),a		;
	call drawbars		;

	ld a,1			; 1=Main menu
	ld (progpos),a		;
	ld a,$1B		;
	ld (barmax),a		;
	xor a			;
	ld (flags),a		;
	ld (varactive),a	;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Main input loop							;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Check for keys... main loop of program
mainloop:
	call drawmenu		; If we're at main menu, draw menu
main_db:
	call drawbar		; And then draw bar...
mainkeyloop:
	call GET_KEY		; -- Try to be TI-85 friendly
	cp K_DOWN		;
	jr z,inclinepos		; Move line down
	cp K_UP			;
	jr z,declinepos		; Move line up...
	cp K_ENTER		;
	jr z,enter_handler	; handle commands
	cp K_LOG		; Display firmware revision
	jp z,disp_rev		; if progpos=1, get firmrev
	cp K_EXIT		;
        jp z,exit_handler	; handle exit key based on progpos
	cp K_MORE		; Did the user hit more?
	jp z,more_handler	;  handle it based on progpos..
	jr mainkeyloop		; Check again

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Key handlers								;
; Most of these can be ported verbatim because the TI-85 has the same	;
; size LCD								;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Increase menu bar position
inclinepos:			; Increase c..
	ld a,(barmax)		; Bar maximum...
	ld c,a			;
	ld a,(barpos)		; barpos=>a
	cp c			; At 2B?
	jr z,mainkeyloop	;  exit

	call drawbar		;
	ld a,(barpos)		;
	add a,$08		; Increase barpos
	ld (barpos),a		;
	call drawbar		; Draw bar
	jr mainkeyloop		; Return to main loop

; Decrease menu bar position
declinepos:
	ld a,(barpos)		; barpos=>a
	cp $0B			; At 0B?
	jr z,mainkeyloop	;  exit

        call drawbar            ;
	ld a,(barpos)		;
	add a,-$08    	        ;
	ld (barpos),a		;
	call drawbar		; Draw bar
	jr mainkeyloop		; Return to main loop

; Ick, jp's are so big :(
enter_handler:
	ld a,(progpos)		;
	cp $01			; Main menu?
	jr z,main_enter		;
	cp $02			; Send to E2 from 86?
	jr z,main_send		;
;	cp $03			; Get from E2 to 86?
;	jr z,main_get		;

main_get:			;
	call read_opt		; Get page number
	ld (page),hl		; Load it
	ld sp,(spbak)		; Restore
	jp get_e2		;
main_send:			;
	call read_opt		; Get mem locat of VAT entry into hl
	ld sp,(spbak)		;
	jp put_e2		; Send to E2...

main_enter:			;
	ld a,(barpos)		; barpos->a
	cp $0B			; List files on TI-86
	jp z,browse86		;
	cp $13			;
	jp z,browsee2		;
	cp $1B			;
	jp z,formate2		;

; Exit program
exit_handler:			; Exit program
	ld a,(progpos)		; 
	cp $01			; If we're at main menu, exit for good
	jr z,exit_program	;
	ld sp,(spbak)		; Restore stack
	jp exitnopause		;

exit_program:
	call CLEARLCD		; Clear screen
	call _flushallmenus	;
	call _homeup		;
	jp _clrScrn		; This'll ret us out

more_handler:			; User wants to see more, take 'em back
	ld a,(progpos)		;
	cp $01			; If they hit more in main menu, treat
	jp z,mainkeyloop	; Ignore on main menu
	ld a,(flags)		;
	or a			; Is a 0?
	jp nz,exitnopause	; Return to main menu, no stack mistakes
	call clearmenu		; Clear workspace
	ld a,(progpos)		;
	cp $03			; Fix / 2047 display bug
	jr z,disp_pgmax		;
resume_dispmax:			;
	ld a,$B			; Reset to B
	ld (disppos),a		;
	pop hl			; Get hl off stack
	jp resume_setpen	;

disp_pgmax:
	call disp_max		;
	jr resume_dispmax	;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Routines								;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Draws top/bottom invert bars...
drawbars:
	call CLEARLCD		; Clear screen
	ld hl,$0009		; Load cursor+row into hl
	ld (_penCol),hl		; load hl into _curRow
	ld hl,eii		; Load address of eii into hl
	call D_ZM_STR		; Display the string
	ld hl,$0037		;
	ld (_penCol),hl		;
	ld hl,bybry		;
	call D_ZM_STR		;
	ld hl,$3914		;
	ld (_penCol),hl		;
	ld hl,url		;
	call D_ZM_STR		;
	ld a,(barpos)		;
	push af			; Backup barpos
	xor a			;
	ld (barpos),a		;
	call drawbar		;
	ld a,$39		;
	ld (barpos),a		;
	call drawbar		;
	pop af			;
	ld (barpos),a		; Restore barpos
	ret			;

; Draws the EII main menu...
drawmenu:
	ld hl,$0B20		;
	ld (_penCol),hl		;
	ld hl,b86		;
	call D_ZM_STR		;
	ld hl,$1320		;
	ld (_penCol),hl		;
	ld hl,be2		;
	call D_ZM_STR		;
	ld hl,$1B20		;
	ld (_penCol),hl		;
	ld hl,forme2		;
	jp D_ZM_STR		; Let it return for us

; Clears menu area but not bars
; Not the smallest or fastest way to do this, but it works
clearmenu:
	ld hl,VIDEO_MEM+$70	;
	ld bc,$316		;
clear_loop:
	ld (hl),0		;
	inc hl			;
	dec bc			;
	ld a,b			;
	or c			;
	jr nz,clear_loop	;
	ret			; ... and cut...

; Invert bar portion of the screen
drawbar:
	ld hl,(barpos)		; Copy barpos into l, crap into h
	ld h,$00		; ... But overwrite crap
	add hl,hl		; Multiply by 16 to get offset
	add hl,hl		;
	add hl,hl		;
	add hl,hl		;
	ld de,VIDEO_MEM		;
	add hl,de		;
	ld b,$70		; b will be a counter -- 7 lines
drawloop:
	ld a,(hl)		; Invert
	xor 255			;
	ld (hl),a		;
	inc hl
	djnz drawloop
	ret

; Setup pencol, handle paging
paging_handler:
	push af
	ld a,(disppos)		; Load display position into a
	cp $33			; Are we at the bottom of things yet?
	jr z,select_var		; Let (l)user chose a variable
resume_setpen:
	pop af			;
	ld bc,(disppos-1)	; Copy it to b...
	ld c,a			; 16 in x (overwrite c from above)
	ld (_penCol),bc		;
	ret			; Return... (just like normal)

; Let user select a variable...
select_var:
	push hl			; Backup hl...
	ld a,$2B		; Assume its a full house...
	ld (barmax),a		;
	xor a			; If this is not last page, clear flags
	ld (flags),a		;
set_barpos:
	ld a,$0B		;
	ld (barpos),a		; Start bar at 10, where text starts
	ld hl,$3203		;
	ld (_penCol),hl		;
	ld hl,varhelp		; Display help line at bottom
	call D_ZM_STR		; Overwrite previous
	jp main_db		; Let 'em play around with x0r bar

bar_a_few:			;
	ld a,(disppos)		; Set bar max at final display pos
	ld (barmax),a		;
	ld a,$01		;
	ld (flags),a		;
	jr set_barpos		;

; Exit routine with clear and a pause
exitpause:
	call wait_key		; Wait for key, return to main
exitnopause:
	call clearmenu		;
	ld a,$01		; Return to main menu...
	ld (progpos),a		;
	ld a,$1B		;
	ld (barmax),a		;
	ld a,$0B		;
	ld (barpos),a		;
	xor a			;
	ld (flags),a		;
	jp mainloop		;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Routine to list files on TI-86				;
; The TI-85 VAT is substantially different... A total rewrite	;
; may be required. :-(						;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
browse86:
	call clearmenu		; Clear menu crap
	call _RAM_PAGE_7	; Rom call to swap to page 7 - VAT
	ld a,$0B		; Start at line ten...
	ld (disppos),a		;
	ld a,$02		; This is program position '2'
	ld (progpos),a		;
	ld hl,$BFC7		; First byte of USER VAT - Rest is system

dir86loop:
	ld a,$30		; Display type at $30
	call paging_handler	; New page pause?
	call update_opt_86	; Update options	
	call disp_vat_entry	;
	ld de,($D298)		; end of vat
	call CP_HL_DE		; Compares hl to de without destroying
	jr z,bar_a_few		;  exit if we're done
	ld a,(disppos)		;
	add a,$8		; Increase it by 8
	ld (disppos),a		;
	jr dir86loop		;

; Displays VAT entry (name, size, type) at current (barpos)
; Takes hl pointer to VAT entry on RAM PAGE 7
disp_vat_entry:
	push hl			; Back our precious mem addr up
	ld a,(hl)		; Load type byte into a
	cp $12			; Is it a program?
	jr z,dispprog           ;   Display program
	cp $0C			; Or a string?
	jr z,dispstng           ;   Display string..
	cp $11			; Or a picture?
	jr z,disppic		;   Display picture..
	cp $04			;
	jr z,dispreal           ; Or a real...
	ld hl,unkn		;
display_vat_type:		; We've displayed type if we recog it.
	call D_ZM_STR		; Display the puppy
				; Now display name+size
	ld hl,buffer1		; Clear 8 bytes from buffer 1
	ld b,9			;
	call _zeromem		;
	pop hl			; Get back our hl
	dec hl			; Skip LSB address byte...
	ld e,(hl)		;
	dec hl			;  MidSB address byte...
	ld d,(hl)		;
	dec hl			;  MostSB addr byte
	ld a,(hl)		;
	push hl			; Store our hl...
	ex de,hl		; Overwrite hl now that its pushed
	call _get_word_ahl	;
	call _RAM_PAGE_7	; Go back to RAM page 7
	pop hl			; Restore our pointer..
	dec hl			; Skip flag byte
	dec hl			; Point at name length byte
	push hl			; Backup for name display routine blow
	ld l,(hl)		; copy name length to hl
	ld h,$00		;
	add hl,de		; Add var size word (from above) to NL
	ld de,$08		; Plus 8 for VAT overhead/size word
	add hl,de		; .. and rest of VAT entry overhead

	ld a,$60		; _penRow already setup
	ld (_penCol),a		;
	call vputHL		; And finally, display entry size

	pop hl			; pop HL...
	call dispname		; Display name
	ret			; We're done, bail

dispprog:
	ld hl,prgm		;
	jr display_vat_type	;

dispstng:
	ld hl,stng		;
	jr display_vat_type	;

dispreal:
	ld hl,real		;
	jr display_vat_type	;

disppic:
	ld hl,pic		;
	jr display_vat_type	;

dispname:
	ld de,buffer1		; Load de with buffer addr
	ld b,(hl)		; Name length
	dec hl			; Back up one byte'
	call dnloop		; b already = name length
	push hl
	ld a,$08		;
	ld (_penCol),a		; And copy to pencol...
	ld hl,buffer1		;
	call D_ZM_STR		; Display string
	pop hl			; Restore our initial hl
	ret			;

; b already = name length
dnloop:
	ld a,(hl)		;
	ld (de),a		;
	inc de			; Can't use leet lddr/ldir :(
	dec hl			; Momos at TI had to have backwards VAT
	djnz dnloop		;
	ret			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Routine to list files on EII					;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
browsee2:
	call clearmenu		;
	call disp_max
	ld a,$0B		; Start at line ten...
	ld (disppos),a		;
	xor a			; Put 0 into file counter
	ld (filecnt),a		;
	ld a,$03		; This is program position '3'
	ld (progpos),a		;
	ld de,0			; Zero counter...
	ld (page),de
list_loop:
; --- Display code ---
	ld hl,$3255		;
	push hl			;
	ld (_penCol),hl		;
	ld hl,space		;
	call D_ZM_STR		; Overwrite previous
	pop hl			;
	ld (_penCol),hl		;
	ld hl,(page)		; Page we're on now...
	call vputHL		; Display it
; --- End display code ---
	ld a,$52		;
	call paging_handler	; Setup display position...
	ld de,(page)		;
	call update_opt_e2	;
	ld hl,buffer1		; Clear buffer
	ld b,9			;
	call _zeromem		;

	call disp_e2_entry	;
	jr z,exit_e2done	;
	ld a,(filecnt)		;
	inc a			;
	ld (filecnt),a		;
	call mv2nextsp		; Move de to next start page
	ld hl,2047		; Did we just scan last page?
	call CP_HL_DE		;
	jr c,exit_liste2	; If the carry is set, page is > 2047
	ld a,(disppos)		;
	add a,$8		;
	ld (disppos),a		;
	jr list_loop

; This is up here so we can jr from above routine
; disp_e2_entry is called, so can be left below..
exit_e2done:
	ld a,(filecnt)		; Did we get any files?!?
	or a			;
	jr z,liste2_none	;
exit_liste2:
	ld a,(disppos)		; Subtract extra bar add-on...
	cp $0B			; Are we at $B?
	jp z,exitnopause	; No more...
	add a,-$08		;
	ld (disppos),a		;
	jp bar_a_few		; Do bar for few filez here

liste2_none:
	ld hl,$1C0F		; Load cursor+row into hl
	ld (_penCol),hl		; load hl into _curRow
	ld hl,nofiles		; Load address of eii into hl
	call D_ZM_STR		; Display the string
	jp exitpause		;

; Checks to see if E2 page has a file start on it...
; If it does, displays line at current disppos. If not, returns with z
; Returns: variable size (non-corrected) in hl
; Takes: _penCol all setup, (page) containing start page
disp_e2_entry:
	ld hl,1			;
	ld (numba),hl		;
	ld a,12			;
	ld (pstart),a		;
	ld hl,temp		;
	call get_sfe		; Get into buffer1 -- Should check for EIIS
	ld a,(temp)		; Is third byte $FF?
	cp $FF			; If it is, this is a blank page...
	ret z			;
	or a			; 92 Program
	jr z,le2_prog92		;
	cp $02			; 92 String
	jr z,le2_stng92		;
	cp $08			; 86 Program
	jr z,le2_prog86		;
	cp $0A			; 86 String
	jr z,le2_stng86		;
	ld b,0			; Don't display calc type
	ld hl,unkn		; Unknown (or rare type we don't care about)
	jr le2d_type		; List e2 display type

le2_prog86:
	ld b,86			; Its an 86 (like it should be)
	jr le2_p2		;
le2_prog92:
	ld b,92			; Its a 92 calculator...
le2_p2:
	ld hl,prgm		;
	jr le2d_type		;

le2_stng86:
	ld b,86			;
	jr le2_s2		;
le2_stng92:
	ld b,92			;
le2_s2:
	ld hl,stng		;

le2d_type:
	push hl			;
	ld h,0			;
	ld l,b			;
	call vputHL		; Display type
	pop hl			; Get string back
	ld a,$30		;
	ld (_penCol),a		;
	call D_ZM_STR		; Display that...
disp_size:
	ld a,$60		; start 8 over
	ld (_penCol),a		;
	call get_e2size		; Get size of this one...
	push hl			; Backup real size for below
	ld bc,$0002		;
	add hl,bc		; Size word is part of program :)
	call vputHL		; Display size....
disp_name:
	ld hl,8			; And now name
	ld (numba),hl		;
	ld a,4			;
	ld (pstart),a		;
	ld hl,buffer1		;
	call get_sfe		;
	ld a,$08		; start 8 over
	ld (_penCol),a		;
	ld hl,buffer1		;
	call D_ZM_STR		;
	pop hl			; Get size back
	or 1			; Clear z
	ret			;

disp_max:
	ld hl,$3266		;
	ld (_penCol),hl		;
disp_max_mypos:			; From own position
	ld hl,pgmax		;
	jp D_ZM_STR		;

; Loads page/mem addr to VAT/E2 variable into opt array below
update_opt_86:
	ex de,hl		; Put mem pos into de, not hl
update_opt_e2:
	push de			;
	ld a,(disppos)		; Load display position
	call calc_array_pos
	ld (hl),e		; Load that value
	inc hl			;
	ld (hl),d		;
	pop hl			; For update_opt_86...
	ret			; And return...

read_opt:
	ld a,(barpos)		;
	call calc_array_pos	;
	call LD_HL_MHL		; ld hl,(hl)
	ret

calc_array_pos:
	add a,-$0B		; Subtract offset
	srl a			; Divide by four
	srl a			;
	ld hl,opt1		; Option 1...
	ld c,a			;
	ld b,0			;
	add hl,bc		; Add offset...
	ret		

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Routine to format EII						;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
formate2:
	call clearmenu
	ld hl,$0C0C		; Load cursor+row into hl
	ld (_penCol),hl		; load hl into _curRow
	ld hl,warn1		; Load address of eii into hl
	call D_ZM_STR		; Display the string
	ld hl,$120C		; Load cursor+row into hl
	ld (_penCol),hl		; load hl into _curRow
	ld hl,warn2		; Load address of eii into hl
	call D_ZM_STR		; Display the string
	call wait_key		;
	cp K_F1			;
	jp nz,exitnopause	; Return
	ld hl,$2045		;
	ld (_penCol),hl		;
	ld hl,pgmax		; Max number of pagez
	call D_ZM_STR		;
	ld de,0			; Start on first page
	ld (page),de		;
form_loop:
	ld hl,(page)		;
	ld de,2048		;
	call CP_HL_DE		;
	jr z,form_done		; Done formatting on page 2047
	ld hl,1			; Get one byet
	ld (numba),hl		;
	xor a			;
	ld (pstart),a		; Start at beginning
	ld hl,temp		; Use buffer1
	call get_sfe		; Check
	ld a,(temp)		; Check it
	cp $FF			; Is it already erased?!
	jr z,skip_to_next	; Slow format -- Checks every page
;	jr z,form_done		; Quick format -- Stops at first $FF page
	call erase_sfe		; Erase that puppy
	call e2_dlay		; Let mem chip program
skip_to_next:
	ld hl,(page)		;
	push hl			;
	ld de,$2030		;
	ld (_penCol),de		;
	call vputHL		;
	pop hl			;
	inc hl			;
	ld (page),hl		;
	jr form_loop		;

form_done:
	ld hl,$2A20		;
	ld (_penCol),hl		;
	ld hl,formdone		; Format done!
	jp disp_pause_exit	; Display, pause, exit, return to main

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Display firmware revision...					;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
disp_rev:
	ld a,(progpos)		; Check semaphore... 
	cp $01			; Only allow from main menu
	jp nz,mainkeyloop	;
	call clearmenu		;
	ld hl,$1C0C		; Load cursor+row into hl
	ld (_penCol),hl		; load hl into _curRow
	ld hl,firmrev		; Load address of eii into hl
	call D_ZM_STR		; Display the string
	ld hl,$1C52		; Setup for number display
	ld (_penCol),hl		;
	call ver_get		; Get version (e2link.asm)
	ld l,a			; Copy to LSByte
	ld h,0			;
	call vputHL		; Display the mubba
	jp exitpause		; Pause, return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; General util routines...					;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Figures out where next program starts in memory
; IN:	de: current page -- will be inc'd
; 	hl: size of data
; OUT:	de: page of next file start
; Assumes: 249 bytes on first page, 260 bytes on every other page
mv2nextsp:			;
	ld de,(page)		;
	or a			; Clear carry
	ld bc,0250		; Amount of data that can fit onto pg1
	sbc hl,bc		; Subtract 250 (To see if =< 249 - 1st page)
	jr c,last_pg_inc	; Increase to next page, only takes 1
	inc hl			; Number of bytes left... 249 on page 1
	inc de			; Increase page counter
pg_inc_loop:			; Increase until we've fit the whole file..
	or a			; Clear carry...
	ld bc,261		; 260 bytes on each data page (EIIC)
	sbc hl,bc		;
	jr c,last_pg_inc	; There are less than 261 bytes left: it'll fit
	inc hl			; We had to subtract one too many...
	inc de			; Move to next page, won't fit...
	jr pg_inc_loop		; Continue until we're done
last_pg_inc:
	inc de			;
	ld (page),de		; Load it into page
	ret

get_e2size:			; Returns size of prog starting on (page)
	ld hl,2			;  in hl
	ld (numba),hl		;
	ld a,13			; Size word
	ld (pstart),a		;
	ld hl,temp		;
	call get_sfe		; Get it...
	ld a,(temp)		; MSByte first..
	ld h,a			; We want big endian, requires a little
	ld a,(temp+1)		; footwork
	ld l,a			;
	ret			;

; The ROM's _getkey handles silent linking - DO NOT USE!!!
wait_key:
	call GET_KEY		; ROM keyscan call
	cp K_NOKEY		;
	jr z,wait_key		;
	ret			;

; Small delay to allow E2 EEPROM to write the page (~20ms)
e2_dlay:
;	push af
;	push bc			;
	ld bc,10000		;
e2_dloop:			;
	dec bc			;
	ld a,c			;
	or b			;
	jr nz,e2_dloop		;
;	pop bc			;
;	pop af			;
	ret			;

; zeromem -- In case your TI-8X doesn't have this ROM call
;zeromem:
;	ld (hl),0		;
;	inc hl			;
;	djnz zeromem		;
;	ret			;

; hl to decimal display routine...
; destroys de
vputHL:				;
	ld de,buffer2+6		;
	xor a			; Clear last char
	ld (de),a		;
	ld b,5			; Only display five characters..
RepUnp:				;
	call UNPACK_HL		;
	or a			; If next digit is a 0, print so far
	jr z,check_rest		;
not_zero:
	dec de			;
	add a,48		;
	ld (de),a		;
	djnz RepUnp		;
disp_it:
	ex de,hl		;
	jp D_ZM_STR		;

; Checks to see if hl is at 00, if it is, go right to disp_it
; If its not, then we just have a normal 0 digit...
check_rest:
	push af			;
	ld a,h			;
	or l			;
	jr nz,check_nz		; We're not done...
	inc sp			; Unfuck stack
	inc sp			;
	jr disp_it		; We are done
check_nz:
	pop af			;
	jr not_zero		; Otherwise, keep going

; Includes for rest of driver
#include "e2link.asm"
#include "e2var.asm"

; Program Text
eii:		.db	"Expand86",0		; Text
bybry:		.db	"By Bryan Rittmeyer",0	; Text
url:		.db	"http://horizon.pair.com/",0
b86:		.db	"Browse TI-86",0
be2:		.db	"Browse E2",0
forme2:		.db	"Format E2",0
;gse2s:		.db	"Get EII Status",0

; Help for VAR select code...
varhelp:	.db	"[ENTER]    [MORE]    [EXIT]",0

; Firmware revision
firmrev:	.db	"Firmware Revision:",0

; Other stuff
to:		.db	"Time Out!",0
nofiles:	.db	"No Files Found!",0
pgmax:		.db	"/  2047",0
space:		.db	"          ",0
warn1:		.db	"Format erases ALL variables.",0
warn2:		.db	"Press F1 to continue",0

stng:		.db	"String",0
prgm:		.db	"Program",0
real:		.db	"Real",0
pic:		.db	"Picture",0
unkn:		.db	"Unknown",0

secs:		.db	"EIIS",0
secc:		.db	"EIIC",0

vsent:		.db	"Variable Sent!",0
vrecd:		.db	"Variable Received!",0
vexists:	.db	"Already Exists!",0
formdone:	.db	"Quick Format Done!",0

onpg:		.db	"Page:",0
vleft:		.db	"Remaining:",0
vsend:		.db	"Sending Variable...",0
vrec:		.db	"Receiving Variable..",0
vunkn:		.db	"Unknown Type!",0
vd:		.db	"(VD)",0

; Temporary storage...
; Its there anyway so we may as well use as much as we need
; (Up to 168 bytes)
workspace	equ	$		; Set to end of program
					; set this to TEXT_MEM on 85

buffer1		equ	workspace	; Use end of program for storage (8B)
buffer2		equ	workspace+9	; Two buffers for 8 byte strings (8B)
barpos		equ	workspace+18	; menu bar position (B)
;vatend		equ 	workspace+19	; Store pointer to end of VAT here (W)
disppos		equ	workspace+21	; Display routine position (B)
;varsize	equ	workspace+22	; Variable size (W)
page		equ	workspace+24	; Page word... (W)
start		equ	workspace+26	; start at byte (B)
numba		equ	workspace+27	; Number of bytes to get word (W)
pstart		equ	workspace+29	; Byte to start at...
filecnt		equ	workspace+30	; File counter (W)
temp		equ	workspace+32	; Temporary word ... (W)
spbak		equ	workspace+34	; Stack backup (W)
barmax		equ	workspace+36	; Maximum bar position (B)
size		equ	workspace+37	; Size... (W)

datptr		equ	workspace+39	; Pointer, goes to 41
varactive	equ	workspace+41

opt1		equ	workspace+100	; Store page/hl ptr to option VAT/E2
opt2		equ	workspace+102	;
opt3		equ	workspace+104	;
opt4		equ	workspace+106	;
opt5		equ	workspace+108	;

progpos		equ	workspace+166	; Program position: what to do
flags		equ	workspace+167	; What to do, second semaphore

; End of program.. wh00p!
.end
